## Project Structure

```
example/
├─README.md
├─firebase.json
└─functions/
  ├─lib/
  ├─node/
  │ └─main.dart
  ├─package.json
  ├─build.yaml
  └─pubspec.yaml
```

## Compile
```bash
firebase init functions
cd functions
pub get
pub run build_runner build --output build
firebase deploy
```

### example/functions/node/main.dart

```dart
import 'package:firebase_node/functions.dart';
import 'package:firebase_node/admin.dart' as admin;

void main() {
  functions['onCreateUser'] =
      functions.auth.user().onCreate((user, context) async {
    final app = await admin.initializeApp();
    final ref = app.firestore().collection('userdata').doc(user.uid);
    await ref.set({
      'name': user.displayName,
      'email': user.email,
      'timestamp': admin.FieldValue.serverTimestamp(),
    });
  });

  functions['onDeleteUser'] = functions.auth.user().onDelete(onDeleteUser);

  functions['storageFunctions'] = functions.storage
      .bucket('my-bucket')
      .object()
      .onFinalize((metadata, context) async {
    final app = admin.initializeApp();
    final ref = app.firestore().collection('tracker').doc('my-bucket');
    final fileName = metadata.name;
    final fileSize = metadata.size;
    final fileId = metadata.id;
    final timestamp =
        admin.Timestamp.fromDateTime(DateTime.parse(metadata.updated));
    final data = {
      'file.$fileId': {
        'name': fileName,
        'size': fileSize,
        'timestamp': timestamp,
      },
      'file_count': admin.FieldValue.increment(1),
    };
    await ref.update(data: data);
  });
}

Future<void> onDeleteUser(UserRecord user, EventContext context) async {
  final app = await admin.initializeApp();
  final ref = app.firestore().collection('userdata').doc(user.uid);
  await ref.delete();
}

```

### example/functions/build.yaml

```yaml
targets:
  $default:
    builders:
      build_node_compilers|entrypoint:
        options:
          compiler: dart2js
          dart2js_args:
            - -O1
```

### example/functions/pubspec.yaml

```yaml
name: firebase_node_example

environment:
  sdk: '>=2.8.1 <3.0.0'

dependencies:
  firebase_node:

dev_dependencies:
  build_node_compilers:
  build_runner:
```

### example/functions/package.json

```json
{
  "name": "firebase_node_example",
  "engines": {
    "node": "8"
  },
  "main": "build/node/main.dart.js",
  "dependencies": {
    "firebase-admin": "^8.10.0",
    "firebase-functions": "^3.6.1"
  }
}
```

### example/firebase.json

```json
{
  "functions": {
    "ignore": ["node_modules", ".dart_tool", "packages"]
  }
}
```
