import '../../admin.dart';
import '../utils/js.dart';
import '../utils/utils.dart';
import 'interop/machine_learning_interop.dart' as interop;

export 'interop/machine_learning_interop.dart'
    show ListModelsOptions, TFLiteModel;

class ListModelsResult extends JsObjectWrapper<interop.ListModelsResultJsImpl>
    implements _ListModelsResult {
  static final _expando = Expando<ListModelsResult>();

  factory ListModelsResult.getInstance(
      interop.ListModelsResultJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= ListModelsResult._fromJsObject(jsObject);
  }

  ListModelsResult._fromJsObject(interop.ListModelsResultJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  List<Model> get models => List.from(jsObject.models);

  @override
  String get pageToken => jsObject.pageToken;
}

class MachineLearning extends JsObjectWrapper<interop.MachineLearningJsImpl>
    implements _MachineLearning {
  static final _expando = Expando<MachineLearning>();

  factory MachineLearning.getInstance(interop.MachineLearningJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= MachineLearning._fromJsObject(jsObject);
  }

  MachineLearning._fromJsObject(interop.MachineLearningJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  App get app => App.getInstance(jsObject.app);

  @override
  Future<void> createModel(ModelOptions model) =>
      handleThenable(jsObject.createModel(model.jsObject));

  @override
  Future<void> deleteModel(String modelId) =>
      handleThenable(jsObject.deleteModel(modelId));

  @override
  Future<Model> getModel(String modelId) =>
      handleThenable(jsObject.getModel(modelId))
          .then((value) => Model.getInstance(value));

  @override
  Future<ListModelsResult> listModels(interop.ListModelsOptions options) =>
      handleThenable(jsObject.listModels(options))
          .then((value) => ListModelsResult.getInstance(value));
  @override
  Future<Model> publishModel(String modelId) =>
      handleThenable(jsObject.publishModel(modelId))
          .then((value) => Model.getInstance(value));

  @override
  Future<Model> unpublishModel(String modelId) =>
      handleThenable(jsObject.unpublishModel(modelId))
          .then((value) => Model.getInstance(value));

  @override
  Future<Model> updateModel(String modelId, ModelOptions model) =>
      handleThenable(jsObject.updateModel(modelId, model.jsObject))
          .then((value) => Model.getInstance(value));
}

class Model extends JsObjectWrapper<interop.ModelJsImpl> implements _Model {
  static final _expando = Expando<Model>();

  factory Model.getInstance(interop.ModelJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= Model._fromJsObject(jsObject);
  }

  Model._fromJsObject(interop.ModelJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  String get createTime => jsObject.createTime;

  @override
  String get displayName => jsObject.displayName;

  @override
  String get etag => jsObject.etag;

  @override
  bool get locked => jsObject.locked;

  @override
  String get modelHash => jsObject.modelHash;

  @override
  String get modelId => jsObject.modelId;

  @override
  bool get published => jsObject.published;

  @override
  List<String> get tags => jsObject.tags;

  @override
  interop.TFLiteModel get tfliteModel => jsObject.tfliteModel;

  @override
  String get updateTime => jsObject.updateTime;

  @override
  String get validationError => jsObject.validationError;

  @override
  Future<void> waitForUnlocked([int maxTimeSeconds]) {
    if (maxTimeSeconds == null) {
      return handleThenable(jsObject.waitForUnlocked());
    } else {
      return handleThenable(jsObject.waitForUnlocked(maxTimeSeconds));
    }
  }
}

class ModelOptions extends JsObjectWrapper<interop.ModelOptionsJsImpl>
    implements _ModelOptions {
  static final _expando = Expando<ModelOptions>();

  factory ModelOptions({
    String displayName,
    List<String> tags,
    interop.TFLiteModel tfliteModel,
  }) {
    return ModelOptions.getInstance(interop.ModelOptionsJsImpl(
      displayName: displayName,
      tfliteModel: tfliteModel,
      tags: tags,
    ));
  }

  factory ModelOptions.getInstance(interop.ModelOptionsJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= ModelOptions._fromJsObject(jsObject);
  }

  ModelOptions._fromJsObject(interop.ModelOptionsJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  String get displayName => jsObject.displayName;

  @override
  set displayName(String v) => jsObject.displayName = v;

  @override
  List<String> get tags => List.from(jsObject.tags);

  @override
  set tags(List<String> v) => jsObject.tags = jsify(v);

  @override
  interop.TFLiteModel get tfliteModel => jsObject.tfliteModel;

  @override
  set tfliteModel(interop.TFLiteModel v) => jsObject.tfliteModel = v;
}

abstract class _ListModelsResult {
  List<Model> get models;
  String get pageToken;
}

abstract class _MachineLearning {
  App get app;
  Future<void> createModel(ModelOptions model);
  Future<void> deleteModel(String modelId);
  Future<Model> getModel(String modelId);
  Future<ListModelsResult> listModels(interop.ListModelsOptions options);
  Future<Model> publishModel(String modelId);
  Future<Model> unpublishModel(String modelId);
  Future<Model> updateModel(String modelId, ModelOptions model);
}

abstract class _Model {
  String get createTime;
  String get displayName;
  String get etag;
  bool get locked;
  String get modelHash;
  String get modelId;
  bool get published;
  List<String> get tags;
  interop.TFLiteModel get tfliteModel;
  String get updateTime;
  String get validationError;
  Future<void> waitForUnlocked([int maxTimeSeconds]);
}

abstract class _ModelOptions {
  String get displayName;
  set displayName(String v);
  List<String> get tags;
  set tags(List<String> v);
  interop.TFLiteModel get tfliteModel;
  set tfliteModel(interop.TFLiteModel v);
}
