import '../utils/js.dart';
import '../utils/utils.dart';
import 'app.dart';
import 'interop/admin_interop.dart';
import 'interop/messaging_interop.dart' as interop;

export 'interop/messaging_interop.dart'
    show
        AndroidFcmOptions,
        ApnsFcmOptions,
        CriticalSound,
        FcmOptions,
        LightSettings,
        MessagingDeviceResult,
        MessagingDevicesResponse,
        MessagingOptions,
        MessagingTopicResponse,
        Notification,
        SendResponse,
        WebPushActions,
        WebpushFcmOptions,
        MessagingConditionResponse;

class AndroidConfig extends JsObjectWrapper<interop.AndroidConfigJsImpl>
    implements _AndroidConfig {
  static final _expando = Expando<AndroidConfig>();

  factory AndroidConfig.getInstance(interop.AndroidConfigJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= AndroidConfig._fromJsObject(jsObject);
  }

  AndroidConfig._fromJsObject(interop.AndroidConfigJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  String get collapseKey => jsObject.collapseKey;

  @override
  set collapseKey(String v) => jsObject.collapseKey = v;

  @override
  Map<String, String> get data {
    final result = dartify(jsObject.data);
    if (result == null) return null;
    return (result as Map<String, dynamic>).cast<String, String>();
  }

  @override
  set data(Map<String, String> v) => jsObject.data = jsify(v);

  @override
  interop.AndroidFcmOptions get fcmOptions => jsObject.fcmOptions;

  @override
  set fcmOptions(interop.AndroidFcmOptions v) => jsObject.fcmOptions = v;

  @override
  AndroidNotification get notification =>
      AndroidNotification.getInstance(jsObject.notification);

  @override
  set notification(AndroidNotification v) => jsObject.notification = v.jsObject;

  @override
  String get priority => jsObject.priority;

  @override
  set priority(String v) => jsObject.priority = v;

  @override
  String get restrictedPackageName => jsObject.restrictedPackageName;

  @override
  set restrictedPackageName(String v) => jsObject.restrictedPackageName = v;

  @override
  num get ttl => jsObject.ttl;

  @override
  set ttl(num v) => jsObject.ttl = v;
}

class AndroidNotification
    extends JsObjectWrapper<interop.AndroidNotificationJsImpl>
    implements _AndroidNotification {
  static final _expando = Expando<AndroidNotification>();

  factory AndroidNotification.getInstance(
      interop.AndroidNotificationJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= AndroidNotification._fromJsObject(jsObject);
  }

  AndroidNotification._fromJsObject(interop.AndroidNotificationJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  String get body => jsObject.body;

  @override
  String get bodyLocArgs => jsObject.bodyLocArgs;

  @override
  String get bodyLocKey => jsObject.bodyLocKey;

  @override
  String get channelId => jsObject.channelId;

  @override
  String get clickAction => jsObject.clickAction;

  @override
  String get color => jsObject.color;

  @override
  bool get defaultLightSettings => jsObject.defaultLightSettings;

  @override
  bool get defaultSound => jsObject.defaultSound;

  @override
  bool get defaultVibrateTimings => jsObject.defaultVibrateTimings;

  @override
  DateTime get eventTimestamp => jsObject.eventTimestamp;

  @override
  String get icon => jsObject.icon;

  @override
  String get imageUrl => jsObject.imageUrl;

  @override
  interop.LightSettings get lightSettings => jsObject.lightSettings;

  @override
  bool get localOnly => jsObject.localOnly;

  @override
  num get notificationCount => jsObject.notificationCount;

  @override
  String get priority => jsObject.priority;

  @override
  String get sound => jsObject.sound;

  @override
  bool get sticky => jsObject.sticky;

  @override
  String get tag => jsObject.tag;

  @override
  String get ticker => jsObject.ticker;

  @override
  String get title => jsObject.title;

  @override
  String get titleLocArgs => jsObject.titleLocArgs;

  @override
  String get titleLocKey => jsObject.titleLocKey;

  @override
  List<num> get vibrateTimingsMillis => jsObject.vibrateTimingsMillis == null
      ? null
      : List.from(jsObject.vibrateTimingsMillis);

  @override
  String get visibility => jsObject.visibility;
}

class ApnsConfig extends JsObjectWrapper<interop.ApnsConfigJsImpl>
    implements _ApnsConfig {
  static final _expando = Expando<ApnsConfig>();

  factory ApnsConfig.getInstance(interop.ApnsConfigJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= ApnsConfig._fromJsObject(jsObject);
  }

  ApnsConfig._fromJsObject(interop.ApnsConfigJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  interop.ApnsFcmOptions get fcmOptions => jsObject.fcmOptions;

  @override
  set fcmOptions(interop.ApnsFcmOptions v) => jsObject.fcmOptions = v;

  @override
  Map<String, String> get headers {
    if (jsObject.headers == null) return null;
    final result = dartify(jsObject.headers);
    return (result as Map<String, dynamic>).cast<String, String>();
  }

  @override
  set headers(Map<String, String> v) => jsObject.headers = jsify(v);

  @override
  ApnsPayload get payload => ApnsPayload.getInstance(jsObject.payload);

  @override
  set payload(ApnsPayload v) => jsObject.payload = v.jsObject;
}

class ApnsPayload extends JsObjectWrapper<interop.ApnsPayloadJsImpl>
    implements _ApnsPayload {
  static final _expando = Expando<ApnsPayload>();

  factory ApnsPayload.getInstance(interop.ApnsPayloadJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= ApnsPayload._fromJsObject(jsObject);
  }

  ApnsPayload._fromJsObject(interop.ApnsPayloadJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  Aps get aps => Aps.getInstance(jsObject.aps);

  @override
  Map<String, dynamic> get customData => dartify(jsObject.customData);
}

class Aps extends JsObjectWrapper<interop.ApsJsImpl> implements _Aps {
  static final _expando = Expando<Aps>();

  factory Aps.getInstance(interop.ApsJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= Aps._fromJsObject(jsObject);
  }

  Aps._fromJsObject(interop.ApsJsImpl jsObject) : super.fromJsObject(jsObject);

  /// alert can be  String | ApsAlert
  @override
  dynamic /*String | ApsAlert*/ get alert => jsObject.alert;

  @override
  num get badge => jsObject.badge;

  @override
  String get category => jsObject.category;

  @override
  bool get contentAvailable => jsObject.contentAvailable;

  @override
  Map<String, dynamic> get customData => dartify(jsObject.customData);

  @override
  bool get mutableContent => jsObject.mutableContent;

  /// sound can be  String | CriticalSound
  @override
  dynamic /*String | CriticalSound*/ get sound => jsObject.sound;

  @override
  String get threadId => jsObject.threadId;
}

class ApsAlert extends JsObjectWrapper<interop.ApsAlertJsImpl>
    implements _ApsAlert {
  static final _expando = Expando<ApsAlert>();

  factory ApsAlert.getInstance(interop.ApsAlertJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= ApsAlert._fromJsObject(jsObject);
  }

  ApsAlert._fromJsObject(interop.ApsAlertJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  String get actionLocKey => jsObject.actionLocKey;

  @override
  String get body => jsObject.body;

  @override
  String get launchImage => jsObject.launchImage;

  @override
  List<String> get locArgs => List.from(jsObject.locArgs);

  @override
  String get locKey => jsObject.locKey;

  @override
  String get subtitle => jsObject.subtitle;

  @override
  List<String> get subtitleLocArgs => List.from(jsObject.subtitleLocArgs);

  @override
  String get subtitleLocKey => jsObject.subtitleLocKey;

  @override
  String get title => jsObject.title;

  @override
  List<String> get titleLocArgs => List.from(jsObject.titleLocArgs);

  @override
  String get titleLocKey => jsObject.titleLocKey;
}

class BatchResponse extends JsObjectWrapper<interop.BatchResponseJsImpl>
    implements _BatchResponse {
  static final _expando = Expando<BatchResponse>();

  factory BatchResponse.getInstance(interop.BatchResponseJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= BatchResponse._fromJsObject(jsObject);
  }

  BatchResponse._fromJsObject(interop.BatchResponseJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  num get failureCount => jsObject.failureCount;

  @override
  List<interop.SendResponse> get responses => jsObject.responses;

  @override
  num get successCount => jsObject.successCount;
}

class ConditionMessage extends JsObjectWrapper<interop.ConditionMessageJsImpl>
    implements _ConditionMessage {
  factory ConditionMessage({
    AndroidConfig android,
    ApnsConfig apns,
    String condition,
    Map<String, dynamic> data,
    interop.FcmOptions fcmOptions,
    interop.Notification notification,
    WebpushConfig webpush,
  }) =>
      ConditionMessage._fromJsObject(interop.ConditionMessageJsImpl(
        android: android.jsObject,
        apns: apns.jsObject,
        condition: condition,
        data: jsify(data),
        fcmOptions: fcmOptions,
        notification: notification,
        webpush: webpush.jsObject,
      ));

  ConditionMessage._fromJsObject(interop.ConditionMessageJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  AndroidConfig get android => AndroidConfig.getInstance(jsObject.android);

  @override
  set android(AndroidConfig v) => jsObject.android = v.jsObject;

  @override
  ApnsConfig get apns => ApnsConfig.getInstance(jsObject.apns);

  @override
  set apns(ApnsConfig v) => jsObject.apns = v.jsObject;

  @override
  String get condition => jsObject.condition;

  @override
  set condition(String v) => jsObject.condition = v;

  @override
  Map<String, dynamic> get data => dartify(jsObject.data);

  @override
  set data(Map<String, dynamic> v) => jsObject.data = jsify(v);

  @override
  interop.FcmOptions get fcmOptions => jsObject.fcmOptions;

  @override
  set fcmOptions(interop.FcmOptions v) => jsObject.fcmOptions = v;

  @override
  interop.Notification get notification => jsObject.notification;

  @override
  set notification(interop.Notification v) => jsObject.notification = v;

  @override
  WebpushConfig get webpush => WebpushConfig.getInstance(jsObject.webpush);

  @override
  set webpush(WebpushConfig v) => jsObject.webpush = v.jsObject;
}

class Messaging extends JsObjectWrapper<interop.MessagingJsImpl>
    implements _Messaging {
  static final _expando = Expando<Messaging>();

  factory Messaging.getInstance(interop.MessagingJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= Messaging._fromJsObject(jsObject);
  }

  Messaging._fromJsObject(interop.MessagingJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  App get app => App.getInstance(jsObject.app);

  @override
  Future<String> send(dynamic message, {bool dryRun = false}) =>
      handleThenable(jsObject.send(message, dryRun));

  @override
  Future<BatchResponse> sendAll(List message, {bool dryRun = false}) =>
      handleThenable(jsObject.sendAll(message, dryRun))
          .then((value) => BatchResponse.getInstance(value));

  @override
  Future<BatchResponse> sendMulticast(MulticastMessage message,
          {bool dryRun = false}) =>
      handleThenable(jsObject.sendMulticast(message.jsObject, dryRun))
          .then((value) => BatchResponse.getInstance(value));

  @override
  Future<interop.MessagingConditionResponse> sendToCondition(
    String condition,
    MessagingPayload payload, [
    interop.MessagingOptions options,
  ]) {
    if (options == null) {
      return handleThenable(
          jsObject.sendToCondition(condition, payload.jsObject));
    } else {
      return handleThenable(
          jsObject.sendToCondition(condition, payload.jsObject, options));
    }
  }

  @override
  Future<interop.MessagingDevicesResponse> sendToDevice(
    List<String> registrationToken,
    MessagingPayload payload, [
    interop.MessagingOptions options,
  ]) {
    if (options == null) {
      return handleThenable(
          jsObject.sendToDevice(jsify(registrationToken), payload.jsObject));
    } else {
      return handleThenable(jsObject.sendToDevice(
          jsify(registrationToken), payload.jsObject, options));
    }
  }

  @override
  Future<MessagingDeviceGroupResponse> sendToDeviceGroup(
    String notificationKey,
    MessagingPayload payload, [
    interop.MessagingOptions options,
  ]) {
    if (options == null) {
      return handleThenable(
              jsObject.sendToDeviceGroup(notificationKey, payload.jsObject))
          .then((value) => MessagingDeviceGroupResponse.getInstance(value));
    } else {
      return handleThenable(jsObject.sendToDeviceGroup(
              notificationKey, payload.jsObject, options))
          .then((value) => MessagingDeviceGroupResponse.getInstance(value));
    }
  }

  @override
  Future<interop.MessagingTopicResponse> sendToTopic(
    String topic,
    MessagingPayload payload, [
    interop.MessagingOptions options,
  ]) {
    if (options == null) {
      return handleThenable(jsObject.sendToTopic(topic, payload.jsObject));
    } else {
      return handleThenable(
          jsObject.sendToTopic(topic, payload.jsObject, options));
    }
  }

  @override
  Future<MessagingTopicManagementResponse> subscribeToTopic(
          List<String> registrationToken, String topic) =>
      handleThenable(jsObject.subscribeToTopic(jsify(registrationToken), topic))
          .then((value) => MessagingTopicManagementResponse.getInstance(value));

  @override
  Future<MessagingTopicManagementResponse> unsubscribeFromTopic(
          List<String> registrationToken, String topic) =>
      handleThenable(
              jsObject.unsubscribeFromTopic(jsify(registrationToken), topic))
          .then((value) => MessagingTopicManagementResponse.getInstance(value));
}

class MessagingDeviceGroupResponse
    extends JsObjectWrapper<interop.MessagingDeviceGroupResponseJsImpl>
    implements _MessagingDeviceGroupResponse {
  static final _expando = Expando<MessagingDeviceGroupResponse>();

  factory MessagingDeviceGroupResponse.getInstance(
      interop.MessagingDeviceGroupResponseJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??=
        MessagingDeviceGroupResponse._fromJsObject(jsObject);
  }

  MessagingDeviceGroupResponse._fromJsObject(
      interop.MessagingDeviceGroupResponseJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  List<String> get failedRegistrationTokens =>
      jsObject.failedRegistrationTokens;

  @override
  num get failureCount => jsObject.failureCount;

  @override
  num get successCount => jsObject.successCount;
}

class MessagingPayload extends JsObjectWrapper<interop.MessagingPayloadJsImpl>
    implements _MessagingPayload {
  factory MessagingPayload(
    Map<String, dynamic> data,
    interop.NotificationMessagePayload notification,
  ) =>
      MessagingPayload._fromJsObject(interop.MessagingPayloadJsImpl(
        data: jsify(data),
        notification: notification,
      ));

  MessagingPayload._fromJsObject(interop.MessagingPayloadJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  Map<String, dynamic> get data => dartify(jsObject.data);

  @override
  set data(Map<String, dynamic> v) => jsObject.data = jsify(v);

  @override
  interop.NotificationMessagePayload get notification => jsObject.notification;

  @override
  set notification(interop.NotificationMessagePayload v) =>
      jsObject.notification = v;
}

class MessagingTopicManagementResponse
    extends JsObjectWrapper<interop.MessagingTopicManagementResponseJsImpl>
    implements _MessagingTopicManagementResponse {
  static final _expando = Expando<MessagingTopicManagementResponse>();

  factory MessagingTopicManagementResponse.getInstance(
      interop.MessagingTopicManagementResponseJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??=
        MessagingTopicManagementResponse._fromJsObject(jsObject);
  }

  MessagingTopicManagementResponse._fromJsObject(
      interop.MessagingTopicManagementResponseJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  List<FirebaseArrayIndexError> get errors => List.from(jsObject.errors);

  @override
  num get failureCount => jsObject.failureCount;

  @override
  num get successCount => jsObject.successCount;
}

class MulticastMessage extends JsObjectWrapper<interop.MulticastMessageJsImpl>
    implements _MulticastMessage {
  factory MulticastMessage({
    AndroidConfig android,
    ApnsConfig apns,
    List<String> tokens,
    Map<String, dynamic> data,
    interop.FcmOptions fcmOptions,
    interop.Notification notification,
    WebpushConfig webpush,
  }) =>
      MulticastMessage._fromJsObject(interop.MulticastMessageJsImpl(
        android: android.jsObject,
        apns: apns.jsObject,
        tokens: jsify(tokens),
        data: jsify(data),
        fcmOptions: fcmOptions,
        notification: notification,
        webpush: webpush.jsObject,
      ));

  MulticastMessage._fromJsObject(interop.MulticastMessageJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  AndroidConfig get android => AndroidConfig.getInstance(jsObject.android);

  @override
  set android(AndroidConfig v) => jsObject.android = v.jsObject;

  @override
  ApnsConfig get apns => ApnsConfig.getInstance(jsObject.apns);

  @override
  set apns(ApnsConfig v) => jsObject.apns = v.jsObject;

  @override
  Map<String, dynamic> get data => dartify(jsObject.data);

  @override
  set data(Map<String, dynamic> v) => jsObject.data = jsify(v);

  @override
  interop.FcmOptions get fcmOptions => jsObject.fcmOptions;

  @override
  set fcmOptions(interop.FcmOptions v) => jsObject.fcmOptions = v;

  @override
  interop.Notification get notification => jsObject.notification;

  @override
  set notification(interop.Notification v) => jsObject.notification = v;

  @override
  List<String> get tokens => List.from(jsObject.tokens);

  @override
  set tokens(List<String> v) => jsObject.tokens = jsify(v);

  @override
  WebpushConfig get webpush => WebpushConfig.getInstance(jsObject.webpush);

  @override
  set webpush(WebpushConfig v) => jsObject.webpush = v.jsObject;
}

class TokenMessage extends JsObjectWrapper<interop.TokenMessageJsImpl>
    implements _TokenMessage {
  factory TokenMessage({
    AndroidConfig android,
    ApnsConfig apns,
    String token,
    Map<String, dynamic> data,
    interop.FcmOptions fcmOptions,
    interop.Notification notification,
    WebpushConfig webpush,
  }) =>
      TokenMessage._fromJsObject(interop.TokenMessageJsImpl(
        android: android.jsObject,
        apns: apns.jsObject,
        token: token,
        data: jsify(data),
        fcmOptions: fcmOptions,
        notification: notification,
        webpush: webpush.jsObject,
      ));

  TokenMessage._fromJsObject(interop.TokenMessageJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  AndroidConfig get android => AndroidConfig.getInstance(jsObject.android);

  @override
  set android(AndroidConfig v) => jsObject.android = v.jsObject;

  @override
  ApnsConfig get apns => ApnsConfig.getInstance(jsObject.apns);

  @override
  set apns(ApnsConfig v) => jsObject.apns = v.jsObject;

  @override
  Map<String, dynamic> get data => dartify(jsObject.data);

  @override
  set data(Map<String, dynamic> v) => jsObject.data = jsify(v);

  @override
  interop.FcmOptions get fcmOptions => jsObject.fcmOptions;

  @override
  set fcmOptions(interop.FcmOptions v) => jsObject.fcmOptions = v;

  @override
  interop.Notification get notification => jsObject.notification;

  @override
  set notification(interop.Notification v) => jsObject.notification = v;

  @override
  String get token => jsObject.token;

  @override
  set token(String v) => jsObject.token = v;

  @override
  WebpushConfig get webpush => WebpushConfig.getInstance(jsObject.webpush);

  @override
  set webpush(WebpushConfig v) => jsObject.webpush = v.jsObject;
}

class TopicMessage extends JsObjectWrapper<interop.TopicMessageJsImpl>
    implements _TopicMessage {
  factory TopicMessage({
    AndroidConfig android,
    ApnsConfig apns,
    String topic,
    Map<String, dynamic> data,
    interop.FcmOptions fcmOptions,
    interop.Notification notification,
    WebpushConfig webpush,
  }) =>
      TopicMessage._fromJsObject(interop.TopicMessageJsImpl(
        android: android.jsObject,
        apns: apns.jsObject,
        topic: topic,
        data: jsify(data),
        fcmOptions: fcmOptions,
        notification: notification,
        webpush: webpush.jsObject,
      ));

  TopicMessage._fromJsObject(interop.TopicMessageJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  AndroidConfig get android => AndroidConfig.getInstance(jsObject.android);

  @override
  set android(AndroidConfig v) => jsObject.android = v.jsObject;

  @override
  ApnsConfig get apns => ApnsConfig.getInstance(jsObject.apns);

  @override
  set apns(ApnsConfig v) => jsObject.apns = v.jsObject;

  @override
  Map<String, dynamic> get data => dartify(jsObject.data);

  @override
  set data(Map<String, dynamic> v) => jsObject.data = jsify(v);

  @override
  interop.FcmOptions get fcmOptions => jsObject.fcmOptions;

  @override
  set fcmOptions(interop.FcmOptions v) => jsObject.fcmOptions = v;

  @override
  interop.Notification get notification => jsObject.notification;

  @override
  set notification(interop.Notification v) => jsObject.notification = v;

  @override
  String get topic => jsObject.topic;

  @override
  set topic(String v) => jsObject.topic = v;

  @override
  WebpushConfig get webpush => WebpushConfig.getInstance(jsObject.webpush);

  @override
  set webpush(WebpushConfig v) => jsObject.webpush = v.jsObject;
}

class WebpushConfig extends JsObjectWrapper<interop.WebpushConfigJsImpl>
    implements _WebpushConfig {
  static final _expando = Expando<WebpushConfig>();

  factory WebpushConfig.getInstance(interop.WebpushConfigJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= WebpushConfig._fromJsObject(jsObject);
  }

  WebpushConfig._fromJsObject(interop.WebpushConfigJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  Map<String, String> get data {
    if (jsObject.data == null) return null;
    final result = dartify(jsObject.data);
    return (result as Map<String, dynamic>).cast<String, String>();
  }

  @override
  interop.WebpushFcmOptions get fcmOptions => jsObject.fcmOptions;

  @override
  Map<String, String> get headers {
    if (jsObject.headers == null) return null;
    final result = dartify(jsObject.headers);
    return (result as Map<String, dynamic>).cast<String, String>();
  }

  @override
  WebpushNotification get notification =>
      WebpushNotification.getInstance(jsObject.notification);
}

class WebpushNotification
    extends JsObjectWrapper<interop.WebpushNotificationJsImpl>
    implements _WebpushNotification {
  static final _expando = Expando<WebpushNotification>();

  factory WebpushNotification.getInstance(
      interop.WebpushNotificationJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= WebpushNotification._fromJsObject(jsObject);
  }

  WebpushNotification._fromJsObject(interop.WebpushNotificationJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  List<interop.WebPushActions> get actions => List.from(jsObject.actions);

  @override
  String get badge => jsObject.badge;

  @override
  String get body => jsObject.body;

  @override
  Map<String, dynamic> get data => dartify(jsObject.data);

  @override
  String get dir => jsObject.dir;

  @override
  String get icon => jsObject.icon;

  @override
  String get image => jsObject.image;

  @override
  String get lang => jsObject.lang;

  @override
  bool get renotify => jsObject.renotify;

  @override
  bool get requireInteraction => jsObject.requireInteraction;

  @override
  bool get silent => jsObject.silent;

  @override
  String get tag => jsObject.tag;

  @override
  num get timestamp => jsObject.timestamp;

  @override
  String get title => jsObject.title;

  @override
  dynamic /*num | List<num>*/ get vibrate => jsObject.vibrate;
}

abstract class _AndroidConfig {
  String collapseKey;

  Map<String, String> data;

  interop.AndroidFcmOptions fcmOptions;

  AndroidNotification notification;

  String priority;

  String restrictedPackageName;

  num ttl;
}

abstract class _AndroidNotification {
  String get body;

  String get bodyLocArgs;

  String get bodyLocKey;

  String get channelId;

  String get clickAction;

  String get color;

  bool get defaultLightSettings;

  bool get defaultSound;

  bool get defaultVibrateTimings;

  DateTime get eventTimestamp;

  String get icon;

  String get imageUrl;

  interop.LightSettings get lightSettings;

  bool get localOnly;

  num get notificationCount;

  String get priority;

  String get sound;

  bool get sticky;

  String get tag;

  String get ticker;

  String get title;

  String get titleLocArgs;

  String get titleLocKey;

  List<num> get vibrateTimingsMillis;

  String get visibility;
}

abstract class _ApnsConfig {
  interop.ApnsFcmOptions fcmOptions;

  Map<String, String> headers;

  ApnsPayload payload;
}

abstract class _ApnsPayload {
  Aps get aps;

  Map<String, dynamic> get customData;
}

abstract class _Aps {
  dynamic /*String | ApsAlert*/ get alert;

  num get badge;

  String get category;

  bool get contentAvailable;

  Map<String, dynamic> get customData;

  bool get mutableContent;

  dynamic /*String | CriticalSound*/ get sound;

  String get threadId;
}

abstract class _ApsAlert {
  String get actionLocKey;

  String get body;

  String get launchImage;

  List<String> get locArgs;

  String get locKey;

  String get subtitle;

  List<String> get subtitleLocArgs;

  String get subtitleLocKey;

  String get title;

  List<String> get titleLocArgs;

  String get titleLocKey;
}

abstract class _BaseMessagee {
  AndroidConfig android;

  ApnsConfig apns;

  Map<String, dynamic> data;

  interop.FcmOptions fcmOptions;

  interop.Notification notification;

  WebpushConfig webpush;
}

abstract class _BatchResponse {
  num get failureCount;

  List<interop.SendResponse> get responses;

  num get successCount;
}

abstract class _ConditionMessage extends _BaseMessagee {
  String condition;
}

abstract class _Messaging {
  App get app;

  Future<String> send(
    dynamic /*TokenMessage | TopicMessage | ConditionMessage*/ message, {
    bool dryRun,
  });

  Future<BatchResponse> sendAll(
    List /*TokenMessage | TopicMessage | ConditionMessage*/ message, {
    bool dryRun,
  });

  Future<BatchResponse> sendMulticast(
    MulticastMessage message, {
    bool dryRun,
  });

  Future<interop.MessagingConditionResponse> sendToCondition(
    String condition,
    MessagingPayload payload, [
    interop.MessagingOptions options,
  ]);

  Future<interop.MessagingDevicesResponse> sendToDevice(
    List<String> registrationToken,
    MessagingPayload payload, [
    interop.MessagingOptions options,
  ]);

  Future<MessagingDeviceGroupResponse> sendToDeviceGroup(
    String notificationKey,
    MessagingPayload payload, [
    interop.MessagingOptions options,
  ]);

  Future<interop.MessagingTopicResponse> sendToTopic(
    String topic,
    MessagingPayload payload, [
    interop.MessagingOptions options,
  ]);

  Future<MessagingTopicManagementResponse> subscribeToTopic(
    List<String> registrationToken,
    String topic,
  );

  Future<MessagingTopicManagementResponse> unsubscribeFromTopic(
    List<String> registrationToken,
    String topic,
  );
}

abstract class _MessagingDeviceGroupResponse {
  List<String> get failedRegistrationTokens;

  num get failureCount;

  num get successCount;
}

abstract class _MessagingPayload {
  Map<String, dynamic> data;

  interop.NotificationMessagePayload notification;
}

abstract class _MessagingTopicManagementResponse {
  List<FirebaseArrayIndexError> get errors;

  num get failureCount;

  num get successCount;
}

abstract class _MulticastMessage extends _BaseMessagee {
  List<String> tokens;
}

abstract class _TokenMessage extends _BaseMessagee {
  String token;
}

abstract class _TopicMessage extends _BaseMessagee {
  String topic;
}

abstract class _WebpushConfig {
  Map<String, String> get data;

  interop.WebpushFcmOptions get fcmOptions;

  Map<String, String> get headers;

  WebpushNotification get notification;
}

abstract class _WebpushNotification {
  List<interop.WebPushActions> get actions;

  String get badge;

  String get body;

  Map<String, dynamic> get data;

  String get dir;

  String get icon;

  String get image;

  String get lang;

  bool get renotify;

  bool get requireInteraction;

  bool get silent;

  String get tag;

  num get timestamp;

  String get title;

  dynamic /*num | List<num>*/ get vibrate;
}
