import '../utils/js.dart';
import '../utils/utils.dart';
import 'interop/project_management_interop.dart' as interop;

export 'interop/project_management_interop.dart'
    show AndroidAppMetadata, AppMetadata, IosAppMetadata, ShaCertificate;

class AndroidApp extends JsObjectWrapper<interop.AndroidAppJsImpl>
    implements _AndroidApp {
  static final _expando = Expando<AndroidApp>();

  factory AndroidApp.getInsance(interop.AndroidAppJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= AndroidApp._fromJsObject(jsObject);
  }

  AndroidApp._fromJsObject(interop.AndroidAppJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  String get appId => jsObject.appId;

  @override
  Future<void> addShaCertificate(interop.ShaCertificate certificate) =>
      handleThenable(jsObject.addShaCertificate(certificate));

  @override
  Future<void> deleteShaCertificate(interop.ShaCertificate certificate) =>
      handleThenable(jsObject.deleteShaCertificate(certificate));

  @override
  Future<String> getConfig() => handleThenable(jsObject.getConfig());

  @override
  Future<interop.IosAppMetadata> getMetadata() =>
      handleThenable(jsObject.getMetadata());

  @override
  Future<interop.ShaCertificate> getShaCertificates() =>
      handleThenable(jsObject.getShaCertificates());

  @override
  Future<void> setDisplayName(String newDisplayName) =>
      handleThenable(jsObject.setDisplayName(newDisplayName));
}

class IosApp extends JsObjectWrapper<interop.IosAppJsImpl> implements _IosApp {
  static final _expando = Expando<IosApp>();

  factory IosApp.getInsance(interop.IosAppJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= IosApp._fromJsObject(jsObject);
  }

  IosApp._fromJsObject(interop.IosAppJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  String get appId => jsObject.appId;

  @override
  Future<String> getConfig() => handleThenable(jsObject.getConfig());

  @override
  Future<interop.IosAppMetadata> getMetadata() =>
      handleThenable(jsObject.getMetadata());

  @override
  Future<void> setDisplayName(String newDisplayName) =>
      handleThenable(jsObject.setDisplayName(newDisplayName));
}

class ProjectManagement extends JsObjectWrapper<interop.ProjectManagementJsImpl>
    implements _ProjectManagement {
  static final _expando = Expando<ProjectManagement>();

  factory ProjectManagement.getInstance(
      interop.ProjectManagementJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= ProjectManagement._fromJsObject(jsObject);
  }

  ProjectManagement._fromJsObject(interop.ProjectManagementJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  AndroidApp androidApp(String appId) =>
      AndroidApp.getInsance(jsObject.androidApp(appId));

  @override
  Future<AndroidApp> createAndroidApp(String packageName, String displayName) =>
      handleThenable(jsObject.createAndroidApp(packageName, displayName))
          .then((value) => AndroidApp.getInsance(value));

  @override
  Future<IosApp> createIosApp(String bundleId, String displayName) =>
      handleThenable(jsObject.createIosApp(bundleId, displayName))
          .then((value) => IosApp.getInsance(value));

  @override
  IosApp iosApp(String appId) => IosApp.getInsance(jsObject.iosApp(appId));

  @override
  Future<List<AndroidApp>> listAndroidApps() async {
    final result = await handleThenable(jsObject.listAndroidApps());
    return result.map((e) => AndroidApp.getInsance(e)).toList();
  }

  @override
  Future<List<interop.AppMetadata>> listAppMetadata() async {
    final result = await handleThenable(jsObject.listAppMetadata());
    return result.cast<interop.AppMetadata>();
  }

  @override
  Future<List<IosApp>> listIosApps() async {
    final result = await handleThenable(jsObject.listAndroidApps());
    return result.map((e) => IosApp.getInsance(e)).toList();
  }

  @override
  Future<void> setDisplayName(String newDisplayName) =>
      handleThenable(jsObject.setDisplayName(newDisplayName));

  @override
  interop.ShaCertificate shaCertificate(String shaHash) =>
      jsObject.shaCertificate(shaHash);
}

abstract class _AndroidApp {
  String get appId;
  Future<void> addShaCertificate(interop.ShaCertificate certificate);
  Future<void> deleteShaCertificate(interop.ShaCertificate certificate);
  Future<String> getConfig();
  Future<interop.ShaCertificate> getShaCertificates();
  Future<interop.IosAppMetadata> getMetadata();
  Future<void> setDisplayName(String newDisplayName);
}

abstract class _IosApp {
  String get appId;
  Future<String> getConfig();
  Future<interop.IosAppMetadata> getMetadata();
  Future<void> setDisplayName(String newDisplayName);
}

abstract class _ProjectManagement {
  AndroidApp androidApp(String appId);
  Future<AndroidApp> createAndroidApp(String packageName, String displayName);
  Future<IosApp> createIosApp(String bundleId, String displayName);
  IosApp iosApp(String appId);
  Future<List<AndroidApp>> listAndroidApps();
  Future<List<interop.AppMetadata>> listAppMetadata();
  interop.ShaCertificate shaCertificate(String shaHash);
  Future<List<IosApp>> listIosApps();
  Future<void> setDisplayName(String newDisplayName);
}
