import 'dart:convert';

import '../utils/js.dart';
import '../utils/utils.dart';
import 'app.dart';
import 'interop/remote_config_interop.dart' as interop;

export 'interop/remote_config_interop.dart'
    show
        ExplicitParameterValue,
        InAppDefaultValue,
        PublishOptions,
        RemoteConfigCondition;

class RemoteConfig extends JsObjectWrapper<interop.RemoteConfigJsImpl>
    implements _RemoteConfig {
  static final _expando = Expando<RemoteConfig>();

  factory RemoteConfig.getInstance(interop.RemoteConfigJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= RemoteConfig._fromJsObject(jsObject);
  }

  RemoteConfig._fromJsObject(interop.RemoteConfigJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  App get app => App.getInstance(jsObject.app);

  @override
  RemoteConfigTemplate createTemplateFromJson(Map<String, dynamic> json) =>
      RemoteConfigTemplate.getInstance(
          jsObject.createTemplateFromJSON(jsonEncode(json)));

  @override
  Future<RemoteConfigTemplate> getTemplate() =>
      handleThenable(jsObject.getTemplate())
          .then((value) => RemoteConfigTemplate.getInstance(value));

  @override
  Future<RemoteConfigTemplate> publishTemplate(RemoteConfigTemplate template,
      [interop.PublishOptions options]) async {
    interop.RemoteConfigTemplateJsImpl result;
    if (options == null) {
      result =
          await handleThenable(jsObject.publishTemplate(template.jsObject));
    } else {
      result = await handleThenable(
          jsObject.publishTemplate(template.jsObject, options));
    }
    return RemoteConfigTemplate.getInstance(result);
  }

  @override
  Future<RemoteConfigTemplate> validateTemplate(
          RemoteConfigTemplate template) =>
      handleThenable(jsObject.validateTemplate(template.jsObject))
          .then((value) => RemoteConfigTemplate.getInstance(value));
}

class RemoteConfigParameter
    extends JsObjectWrapper<interop.RemoteConfigParameterJsImpl>
    implements _RemoteConfigParameter {
  static final _expando = Expando<RemoteConfigParameter>();

  @override
  Map<String, dynamic> get conditionalValues {
    if (jsObject.conditionalValues == null) return null;
    return dartify(jsObject.conditionalValues);
  }

  @override
  set conditionalValues(Map<String, dynamic> v) {
    jsObject.conditionalValues = jsify(v);
  }

  @override
  interop.ExplicitParameterValue get defaultValue => jsObject.defaultValue;

  @override
  set defaultValue(interop.ExplicitParameterValue _defaultValue) {
    jsObject.defaultValue = _defaultValue;
  }

  @override
  String get description => jsObject.description;

  @override
  set description(String v) => jsObject.description = v;

  factory RemoteConfigParameter.getInstance(
      interop.RemoteConfigParameterJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= RemoteConfigParameter._fromJsObject(jsObject);
  }

  RemoteConfigParameter._fromJsObject(
      interop.RemoteConfigParameterJsImpl jsObject)
      : super.fromJsObject(jsObject);
}

class RemoteConfigParameterGroup
    extends JsObjectWrapper<interop.RemoteConfigParameterGroupJsImpl>
    implements _RemoteConfigParameterGroup {
  static final _expando = Expando<RemoteConfigParameterGroup>();

  @override
  String get description => jsObject.description;

  @override
  set description(String v) => jsObject.description = v;

  @override
  Map<String, RemoteConfigParameter> get parameters {
    if (jsObject.parameters == null) return null;
    final groups = dartify(dartifyMap(jsObject.parameters)) as Map;
    return groups.cast<String, RemoteConfigParameter>();
  }

  @override
  set parameters(Map<String, RemoteConfigParameter> v) {
    jsObject.parameters = jsify(v);
  }

  factory RemoteConfigParameterGroup.getInstance(
      interop.RemoteConfigParameterGroupJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??=
        RemoteConfigParameterGroup._fromJsObject(jsObject);
  }

  RemoteConfigParameterGroup._fromJsObject(
      interop.RemoteConfigParameterGroupJsImpl jsObject)
      : super.fromJsObject(jsObject);
}

class RemoteConfigTemplate
    extends JsObjectWrapper<interop.RemoteConfigTemplateJsImpl>
    implements _RemoteConfigTemplate {
  static final _expando = Expando<RemoteConfigTemplate>();

  @override
  List<interop.RemoteConfigCondition> get conditions =>
      List.from(jsObject.conditions);

  @override
  set conditions(List<interop.RemoteConfigCondition> v) {
    jsObject.conditions = v;
  }

  @override
  String get etag => jsObject.etag;

  @override
  set etag(String v) => jsObject.etag = v;

  @override
  Map<String, RemoteConfigParameterGroup> get parameterGroups {
    if (jsObject.parameterGroups == null) return null;
    final groups = dartify(dartifyMap(jsObject.parameterGroups)
        .cast<String, interop.RemoteConfigParameterGroupJsImpl>()) as Map;
    return groups.cast<String, RemoteConfigParameterGroup>();
  }

  @override
  set parameterGroups(Map<String, RemoteConfigParameterGroup> v) {
    jsObject.parameterGroups = jsify(v);
  }

  @override
  Map<String, RemoteConfigParameter> get parameters {
    if (jsObject.parameters == null) return null;
    final groups = dartify(dartifyMap(jsObject.parameters)
        .cast<String, interop.RemoteConfigParameterJsImpl>()) as Map;
    return groups.cast<String, RemoteConfigParameter>();
  }

  @override
  set parameters(Map<String, RemoteConfigParameter> v) {
    jsObject.parameters = jsify(v);
  }

  factory RemoteConfigTemplate.getInstance(
      interop.RemoteConfigTemplateJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= RemoteConfigTemplate._fromJsObject(jsObject);
  }

  RemoteConfigTemplate._fromJsObject(
      interop.RemoteConfigTemplateJsImpl jsObject)
      : super.fromJsObject(jsObject);
}

abstract class _RemoteConfig {
  App get app;
  RemoteConfigTemplate createTemplateFromJson(Map<String, dynamic> json);
  Future<RemoteConfigTemplate> getTemplate();
  Future<RemoteConfigTemplate> publishTemplate(RemoteConfigTemplate template,
      [interop.PublishOptions options]);
  Future<RemoteConfigTemplate> validateTemplate(RemoteConfigTemplate template);
}

abstract class _RemoteConfigParameter {
  Map<String, dynamic> conditionalValues;
  interop.ExplicitParameterValue defaultValue;
  String description;
}

abstract class _RemoteConfigParameterGroup {
  String description;
  Map<String, RemoteConfigParameter> parameters;
}

abstract class _RemoteConfigTemplate {
  List<interop.RemoteConfigCondition> conditions;
  String etag;
  Map<String, RemoteConfigParameterGroup> parameterGroups;
  Map<String, RemoteConfigParameter> parameters;
}
