import '../utils/js.dart';
import '../utils/utils.dart';
import 'app.dart';
import 'interop/security_rules_interop.dart' as interop;

class SecurityRules extends JsObjectWrapper<interop.SecurityRulesJsImpl>
    implements _SecurityRules {
  static final _expando = Expando<SecurityRules>();

  factory SecurityRules.getInstance(interop.SecurityRulesJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= SecurityRules._fromJsObject(jsObject);
  }

  SecurityRules._fromJsObject(interop.SecurityRulesJsImpl jsObject)
      : super.fromJsObject(jsObject);

  @override
  App get app => App.getInstance(jsObject.app);

  @override
  Future<interop.Ruleset> createRuleset(interop.RulesFile file) =>
      handleThenable(jsObject.createRuleset(file));

  @override
  interop.RulesFile createRulesFileFromSource(String name, String source) =>
      jsObject.createRulesFileFromSource(name, source);

  @override
  Future<void> deleteRuleset(String name) =>
      handleThenable(jsObject.deleteRuleset(name));

  @override
  Future<interop.Ruleset> getFirestoreRuleset() =>
      handleThenable(jsObject.getFirestoreRuleset());

  @override
  Future<interop.Ruleset> getRuleset(String name) =>
      handleThenable(jsObject.getRuleset(name));

  @override
  Future<interop.Ruleset> getStorageRuleset([String bucket]) {
    if (bucket == null) return handleThenable(jsObject.getStorageRuleset());
    return handleThenable(jsObject.getStorageRuleset(bucket));
  }

  @override
  Future<interop.RulesetMetadataList> listRulesetMetadata(
          [num pageSize, String nextPageToken]) =>
      handleThenable(jsObject.listRulesetMetadata(pageSize, nextPageToken));

  @override
  Future<void> releaseFirestoreRuleset(dynamic ruleset) =>
      handleThenable(jsObject.releaseFirestoreRuleset(ruleset));

  @override
  Future<interop.Ruleset> releaseFirestoreRulesetFromSource(String source) =>
      handleThenable(jsObject.releaseFirestoreRulesetFromSource(source));

  @override
  Future<void> releaseStorageRuleset(dynamic ruleset, [String bucket]) {
    if (bucket == null) {
      return handleThenable(jsObject.releaseStorageRuleset(ruleset));
    } else {
      return handleThenable(jsObject.releaseStorageRuleset(ruleset, bucket));
    }
  }

  @override
  Future<void> releaseStorageRulesetFromSource(dynamic ruleset,
      [String bucket]) {
    if (bucket == null) {
      return handleThenable(jsObject.releaseStorageRulesetFromSource(ruleset));
    } else {
      return handleThenable(
          jsObject.releaseStorageRulesetFromSource(ruleset, bucket));
    }
  }
}

abstract class _SecurityRules {
  App get app;
  Future<interop.Ruleset> createRuleset(interop.RulesFile file);
  interop.RulesFile createRulesFileFromSource(String name, String source);
  Future<void> deleteRuleset(String name);
  Future<interop.Ruleset> getFirestoreRuleset();
  Future<interop.Ruleset> getRuleset(String name);
  Future<interop.Ruleset> getStorageRuleset([String bucket]);
  Future<interop.RulesetMetadataList> listRulesetMetadata(
      [num pageSize, String nextPageToken]);
  Future<void> releaseFirestoreRuleset(
      /*String | RulesetMetadata */ dynamic ruleset);
  Future<interop.Ruleset> releaseFirestoreRulesetFromSource(String source);
  Future<void> releaseStorageRuleset(
    /*String | RulesetMetadata */ dynamic ruleset, [
    String bucket,
  ]);
  Future<void> releaseStorageRulesetFromSource(
    /*String | RulesetMetadata */ dynamic ruleset, [
    String bucket,
  ]);
}
