import 'package:js/js.dart';

import '../admin/auth.dart';
import '../utils/js.dart';
import 'functions.dart';
import 'interop/auth_interop.dart' as interop;

export '../admin/auth.dart' show UserRecord;
export 'interop/auth_interop.dart' show UserRecordMetadata;

class AuthFunctionsBuilder
    extends JsObjectWrapper<interop.AuthFunctionsBuilderJsImpl> {
  AuthFunctionsBuilder.fromJsObject(interop.AuthFunctionsBuilderJsImpl jsObject)
      : super.fromJsObject(jsObject);

  ///Handle events related to Firebase authentication users.
  UserBuilder user() => UserBuilder.getInstance(jsObject.user());
}

class UserBuilder extends JsObjectWrapper<interop.UserBuilderJsImpl> {
  static final _expando = Expando<UserBuilder>();

  factory UserBuilder.getInstance(interop.UserBuilderJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= UserBuilder._fromJsObject(jsObject);
  }

  UserBuilder._fromJsObject(interop.UserBuilderJsImpl jsObject)
      : super.fromJsObject(jsObject);

  ///Respond to the creation of a Firebase Auth user.
  dynamic onCreate(DartHandler<UserRecord> handler) {
    return jsObject.onCreate(allowInterop((dataJs, contextJs) {
      return cloudFunction(
        UserRecord.getInstance(dataJs),
        EventContext.getInstance(contextJs),
        handler,
      );
    }));
  }

  ///Respond to the deletion of a Firebase Auth user.
  dynamic onDelete(DartHandler<UserRecord> handler) {
    // return cloudFunction(handler, jsObject.onCreate, UserRecord.getInstance);
  }
}
