import 'package:js/js.dart';

import '../admin/firestore.dart';
import '../utils/js.dart';
import 'functions.dart';
import 'interop/firestore_interop.dart' as interop;

export '../admin/firestore.dart' show DocumentSnapshot;

class DocumentBuilder extends JsObjectWrapper<interop.DocumentBuilderJsImpl> {
  static final _expando = Expando<DocumentBuilder>();

  factory DocumentBuilder.getInstance(interop.DocumentBuilderJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??= DocumentBuilder._fromJsObject(jsObject);
  }

  DocumentBuilder._fromJsObject(interop.DocumentBuilderJsImpl jsObject)
      : super.fromJsObject(jsObject);

  ///Respond only to document creations.
  dynamic onCreate(DartHandler<DocumentSnapshot> handler) {
    return jsObject.onCreate(allowInterop((snapshotJs, contextJs) {
      return cloudFunction(
        DocumentSnapshot.getInstance(snapshotJs),
        EventContext.getInstance(contextJs),
        handler,
      );
    }));
  }

  ///Respond only to document deletions.
  dynamic onDelete(DartHandler<DocumentSnapshot> handler) {
    return jsObject.onDelete(allowInterop((snapshotJs, contextJs) {
      return cloudFunction(
        DocumentSnapshot.getInstance(snapshotJs),
        EventContext.getInstance(contextJs),
        handler,
      );
    }));
  }

  ///Respond only to document updates.
  dynamic onUpdate(DartHandler<Change<DocumentSnapshot>> handler) {
    return jsObject.onUpdate(allowInterop((dataJs, contextJs) {
      final data = Change(
        after: DocumentSnapshot.getInstance(dataJs.after),
        before: DocumentSnapshot.getInstance(dataJs.before),
      );
      final context = EventContext.getInstance(contextJs);
      return cloudFunction(data, context, handler);
    }));
  }

  ///Respond to all document writes (creates, updates, or deletes).
  dynamic onWrite(DartHandler<Change<DocumentSnapshot>> handler) {
    return jsObject.onWrite(allowInterop((dataJs, contextJs) {
      final data = Change(
        after: DocumentSnapshot.getInstance(dataJs.after),
        before: DocumentSnapshot.getInstance(dataJs.before),
      );
      final context = EventContext.getInstance(contextJs);
      return cloudFunction(data, context, handler);
    }));
  }
}

class FirestoreFunctionsBuilder
    extends JsObjectWrapper<interop.FirestoreFunctionsBuilderJsImpl> {
  static final _expando = Expando<FirestoreFunctionsBuilder>();

  factory FirestoreFunctionsBuilder.getInstance(
      interop.FirestoreFunctionsBuilderJsImpl jsObject) {
    if (jsObject == null) return null;
    return _expando[jsObject] ??=
        FirestoreFunctionsBuilder._fromJsObject(jsObject);
  }

  FirestoreFunctionsBuilder._fromJsObject(
      interop.FirestoreFunctionsBuilderJsImpl jsObject)
      : super.fromJsObject(jsObject);

  ///Select the Firestore document to listen to for events.
  DocumentBuilder document(String path) =>
      DocumentBuilder.getInstance(jsObject.document(path));
}
