import 'dart:async';

import 'package:js/js.dart';
import 'package:meta/meta.dart';
import 'package:node_interop/http.dart';

import '../utils/js.dart';
import '../utils/utils.dart';
import 'express_http.dart';
import 'functions.dart';
import 'interop/https_interop.dart' as interop;

typedef HttpsOnCall = Function(Map<String, dynamic>, CallableContext);
typedef HttpsOnRequest = Function(ExpressHttpRequest);

class CallableContext extends JsObjectWrapper<interop.CallableContextJsImpl> {
  CallableContext._fromJsObject(interop.CallableContextJsImpl jsObject)
      : super.fromJsObject(jsObject);

  ///The result of decoding and verifying a Firebase Auth ID token.
  AuthContext get auth => AuthContext.getInstance(jsObject.auth);

  ///An unverified token for a Firebase Instance ID.
  String get instanceIdToken => jsObject.instanceIdToken;

  ///The raw request handled by the callable.
  IncomingMessage get rawRequest => jsObject.rawRequest;
}

// enum FunctionsErrorCode {
//   ok,
//   cancelled,
//   unknown,
//   invalidArgument,
//   deadlineExceeded,
//   notFound,
//   alreadyExists,
//   permissionDenied,
//   resourceExhausted,
//   failedPrecondition,
//   aborted,
//   outOfRange,
//   unimplemented,
//   internal,
//   unavailable,
//   dataLoss,
//   unauthenticated,
// }

class HttpsError extends JsObjectWrapper<interop.HttpsErrorJsImpl> {
  factory HttpsError({
    @required String code,
    @required Map<String, dynamic> details,
    @required String message,
    @required String name,
    String stack,
  }) {
    if (stack == null) {
      return HttpsError._fromJsObject(interop.HttpsErrorJsImpl(
        code: code,
        details: jsify(details),
        message: message,
        name: name,
      ));
    } else {
      return HttpsError._fromJsObject(interop.HttpsErrorJsImpl(
        code: code,
        details: jsify(details),
        message: message,
        name: name,
        stack: stack,
      ));
    }
  }

  HttpsError._fromJsObject(interop.HttpsErrorJsImpl jsObject)
      : super.fromJsObject(jsObject);

  ///A standard error code that will be returned to the client. This also
  ///determines the HTTP status code of the response, as defined in code.proto.
  String get code => jsObject.code;

  ///Extra data to be converted to JSON and included in the error response.
  Map<String, dynamic> get details => dartify(jsObject.details);

  String get message => jsObject.message;

  String get name => jsObject.name;

  String get stack => jsObject.stack;
}

class HttpsFunctionsBuilder
    extends JsObjectWrapper<interop.HttpsFunctionsBuilderJsImpl> {
  HttpsFunctionsBuilder.fromJsObject(
      interop.HttpsFunctionsBuilderJsImpl jsObject)
      : super.fromJsObject(jsObject);

  ///Declares a callable method for clients to call using a Firebase SDK.
  dynamic onCall(HttpsOnCall handler) =>
      jsObject.onCall(allowInterop((dataJsImpl, contextJsImpl) {
        final data = dartify(dataJsImpl);
        final context = CallableContext._fromJsObject(contextJsImpl);
        final result = handler(data, context);
        if (result is Future) {
          return handleFutureWithMapper(result, jsify);
        } else {
          return jsify(result);
        }
      }));

  ///Handle HTTP requests.
  dynamic onRequest(HttpsOnRequest handler) =>
      jsObject.onRequest(allowInterop((req, res) {
        final result = handler(ExpressHttpRequest(req, res));
        if (result is Future) {
          return handleFutureWithMapper(result, jsify);
        } else {
          return jsify(result);
        }
      }));
}
